package smila.hackatlon.chansonnier.datamodel;

import java.util.Date;

import org.eclipse.smila.datamodel.Any;
import org.eclipse.smila.datamodel.AnyMap;
import org.eclipse.smila.datamodel.AnySeq;
import org.eclipse.smila.datamodel.Record;
import org.eclipse.smila.datamodel.Value;

public abstract class CubrikRecord{
	public enum Field{
		source;
	}
	
	/**
	 * A marker interface to identify the useful enum used into Record wrapper
	 * @author Lorenzo Eccher, Eng
	 *
	 */
	public abstract interface IMarker{
		/**
		 * This method is used to read the name of enum value
		 * @return the enum name
		 */
		public abstract String name();
	}

	/**
	 * This interface is designed to be used into Enum to identify the attachments into metadata {@link AnyMap} of {@link Record}
	 * @author Lorenzo Eccher, Eng
	 *
	 */
	public abstract interface IAttachment extends IMarker{
	}
	/**
	 * This interface is designed to be used into Enum to identify the fields into metadata {@link AnyMap} of {@link Record}
	 * @author Lorenzo Eccher, Eng
	 *
	 */
	public abstract interface IField extends IMarker{
	}

	protected final Record record;

	public CubrikRecord(Record r){
		record = r;
		record.getMetadata().put(Field.source.name(), getSourceName());
	}
	
	/**
	 * Put an entry in metadata member of {@link Record} object with {@link String} value into a synchronized section on given {@link Record} object
	 * @param key the {@link IField} key
	 * @param value The {@link String} value
	 */
	public void put(IField key, String value){
		synchronized (record) {
			record.getMetadata().put(key.name(),value);	
		}
	}
	
	/**
	 * Put an entry in metadata member of {@link Record} object with {@link Boolean} value into a synchronized section on given {@link Record} object
	 * @param key the {@link IField} key
	 * @param value The {@link Boolean} value
	 */
	public void put(IField key, Boolean value){
		synchronized (record) {
			record.getMetadata().put(key.name(),value);	
		}
	}

	/**
	 * Put an entry in metadata member of {@link Record} object with {@link Number} value into a synchronized section on given {@link Record} object 
	 * @param key the {@link IField} key
	 * @param value The {@link Number} value
	 */
	public void put(IField key, Number value){
		synchronized (record) {
			record.getMetadata().put(key.name(),value);	
		}
	}
	
	/**
	 * Put an entry in metadata member of {@link Record} object with {@link Any} value 
	 * @param key the {@link IField} key
	 * @param value The {@link Any} value
	 */
	public void put(IField key, Any value){
		synchronized (record) {
			record.getMetadata().put(key.name(),value);	
		}
	}
	
	/**
	 * @see #getString(smila.hackatlon.chansonnier.datamodel.RecordInterface.IField)
	 */
	public String get(IField key){
		return getString(key);
	}
	/**
	 * @see {@link AnyMap#getStringValue(String)} 
	 * @param key the {@link IField} key
	 */
	public String getString(IField key){
		return record.getMetadata().getStringValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getDouble(String)} 
	 * @param key the {@link IField} key
	 */
	public Double getDouble(IField key){
		return record.getMetadata().getDoubleValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getLong(String)} 
	 * @param key the {@link IField} key
	 */
	public Long getLong(IField key){
		return record.getMetadata().getLongValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getDate(String)} 
	 * @param key the {@link IField} key
	 */
	public Date getDate(IField key){
		return record.getMetadata().getDateValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getDateTime(String)} 
	 * @param key the {@link IField} key
	 */
	public Date getDateTime(IField key){
		return record.getMetadata().getDateTimeValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getBooleanValue(String)} 
	 * @param key the {@link IField} key
	 */
	public Boolean getBoolean(IField key){
		return record.getMetadata().getBooleanValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getValue(String)} 
	 * @param key the {@link IField} key
	 */
	public Value getValue(IField key){
		return record.getMetadata().getValue(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getMap(String)} 
	 * @param key the {@link IField} key
	 */
	public AnyMap getMap(IField key){
		return record.getMetadata().getMap(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getMap(String, boolean)} 
	 * @param key the {@link IField} key
	 */
	public AnyMap getMap(IField key,boolean create){
		return record.getMetadata().getMap(key.name(),create);
	}
	/**
	 * @see {@link AnyMap#getSeq(String)} 
	 * @param key the {@link IField} key
	 */
	public AnySeq getSeq(IField key){
		return record.getMetadata().getSeq(key.name());
	}
	
	/**
	 * @see {@link AnyMap#getSeq(String)} 
	 * @param key the {@link IField} key
	 */
	public AnySeq getSeq(IField key,boolean create){
		return record.getMetadata().getSeq(key.name(),create);
	}
	
	/**
	 * Retrieve the byte[] stored in an attachment into {@link Record} object into a synchronized section on given {@link Record} object 
	 * @param key the {@link IAttachment} key
	 */
	public byte[] get(IAttachment key){
		return record.getAttachment(key.name());
	}

	/**
	 * Put an attachment into {@link Record} object into a synchronized section on given {@link Record} object
	 * @param key the {@link IAttachment} key
	 * @param value The byte[] attachment value
	 */
	public void put(IAttachment key, byte[] value){
		synchronized (record) {
			record.setAttachment(key.name(),value);
		}
	}
	
	protected abstract void fillRecord();
	
	public abstract String getSourceName();
	
}
